#ifndef TESTVIEW_H
#define TESTVIEW_H


#include <QtGui>
#include <QtDeclarative>
#include <QtTest>
#include "qmlapplicationviewer.h"
#include <private/qdeclarativeflickable_p.h>
#include <private/qdeclarativemousearea_p.h>

Q_DECLARE_METATYPE(QList<int>)

class TestView : public QObject
{
    Q_OBJECT
private slots:
    void mouse3Moves_data() {
        QTest::addColumn< QList<int> >("points");
        QTest::newRow("good-1-move") << (QList<int> () << 3);
        QTest::newRow("good-1-move") << (QList<int> () << 4);
        QTest::newRow("bad-1-move") << (QList<int> () << 5);
        QTest::newRow("bad-1-move") << (QList<int> () << 6);
        QTest::newRow("bad-1-move") << (QList<int> () << 12);
        QTest::newRow("good-3-moves") << (QList<int> () << 1 << 3 << 4);
        QTest::newRow("bad-3-moves") << (QList<int> () << 1 << 3 << 5);
        QTest::newRow("bad-3-moves") << (QList<int> () << 1 << 3 << 12);
        QTest::newRow("bad-3-moves") << (QList<int> () << 1 << 4 << 6);
        QTest::newRow("good-3-moves") << (QList<int> () << 1 << 5 << 6);
    }

    void mouse3Moves() {
        qDebug() << "... startDragDistance:" << QApplication::startDragDistance();

        QFETCH(QList<int>, points);

        QmlApplicationViewer *canvas = new QmlApplicationViewer;
        canvas->setMainQmlFile("qml/flickableissue/main.qml");
        canvas->show();
        canvas->setFocus();

        QVERIFY(canvas->rootObject() != 0);
        QDeclarativeMouseArea *mousearea = canvas->rootObject()->findChild<QDeclarativeMouseArea*>("mousearea");
        QVERIFY(mousearea != 0);

        // 1st click with moves
        QPoint origin(25, 25);
        QPoint point = origin;
        QTest::mousePress(canvas->viewport(), Qt::LeftButton, 0, canvas->mapFromScene(point));
        foreach(int deltaX, points) {
            point = origin + QPoint(deltaX, 0);
            qDebug() << "... move:" << deltaX << "delta: " << (point-origin);
            QMouseEvent mv(QEvent::MouseMove, canvas->mapFromScene(point), Qt::LeftButton, Qt::LeftButton,Qt::NoModifier);
            QApplication::sendEvent(canvas->viewport(), &mv);
        }

        QTest::mouseRelease(canvas->viewport(), Qt::LeftButton, 0, canvas->mapFromScene(point));

        QDeclarativeItem* grabber = static_cast<QDeclarativeItem*>(canvas->scene()->mouseGrabberItem());
        //QVERIFY2(grabber == 0, "After release, there should be no grabber");
        if(grabber) {
            qWarning() << "Should there be a grabber after mouse-release? grabber=" << grabber->objectName();
        }

        // 2nd click

        QSignalSpy pressSpy(mousearea, SIGNAL(pressed(QDeclarativeMouseEvent*)));
        QSignalSpy releaseSpy(mousearea, SIGNAL(released(QDeclarativeMouseEvent*)));
        QSignalSpy clickSpy(mousearea, SIGNAL(clicked(QDeclarativeMouseEvent*)));

        QTest::mousePress(canvas->viewport(), Qt::LeftButton, 0, canvas->mapFromScene(QPoint(25, 25)));
        QCOMPARE(pressSpy.count(),1);
        QTest::mouseRelease(canvas->viewport(), Qt::LeftButton, 0, canvas->mapFromScene(QPoint(25, 25)));
        QCOMPARE(releaseSpy.count(),1);

        QCOMPARE(clickSpy.count(),1);

        delete canvas;

    }
};

QTEST_MAIN(TestView)
#include "main.moc"

#endif // TESTVIEW_H
